/**         @file  modmgr.h
 *         @brief  Module manager headers
 *          @date  25/11/2017
 *        @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *     @copyright  Copyright (c) 2017, François Cerbelle
 *
 * Manage module loading with parametrized initialization, module unloading,
 * fetching modules ABI...
 *
 *   @internal
 *       Compiler  gcc
 *  Last modified  2024-07-02 09:51
 *   Organization  Cerbelle.net
 *        Company  Home
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __MODMGR_H__
#define __MODMGR_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdint.h>
#include <ltdl.h>

#include "module.h"

#define MODMGR_GETFUNCTION(m,f) *(void**)(&f)=modmgr_getsymbol(m,#f);
#define MODMGR_LOAD(module,api,filename) \
        module=modmgr_load(filename); \
        api=modmgr_getsymbol(module,"module_api");

#ifdef __cplusplus
extern "C" {
#endif

/** Pointer type on private structure */
typedef struct modules_s *modmgr_module_t;

/** Initialize or reset module search path */
int modmgr_setpath (const char* path);

/** Add a path to the end of the module search path */
int modmgr_addpath (const char* path);

/** Insert an higher priority search path before another one */
int modmgr_insertpath (const char* before, const char* path);

/** Get the current search path list */
const char* modmgr_getpath ();

/** Load a module and call the initialization with a parameter if first usage */
modmgr_module_t modmgr_load(const char* modfile);

/** Call the unload function if last usage and tries to unload the module */
void modmgr_unload(modmgr_module_t module);

/** Print the currently loaded modules list for debug and tracing */
void modmgr_list();

/** Resolve a module symbol, can be a function or a variable */
void* modmgr_getsymbol(const modmgr_module_t module, const char* szSymbol);

#ifdef __cplusplus
}
#endif

#endif /* __MODMGR_H__ */
