/**         @file  svcmgr.h
 *         @brief  Service manager header
 *          @date  27/11/2017
 *        @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *     @copyright  Copyright (c) 2017, François Cerbelle
 *
 *  Defines a service dictionnary or registry in which a module can register
 *  and call a new service, with an endpoint, a callback function and an arbitrary number of arguments.
 *
 *   @internal
 *       Compiler  gcc
 *  Last modified  2024-07-28 19:54
 *   Organization  Cerbelle.net
 *        Company  Home
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __SVCMGR_H__
#define __SVCMGR_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdint.h>
#include <stdarg.h> /* va_list, va_start, va_arg, va_end */

#ifdef __cplusplus
extern "C" {
#endif

typedef uint8_t svcfunc_t (va_list p_ap);

void svcmgr_register(const char* p_endpoint, svcfunc_t* p_service);
uint8_t svcmgr_call(const char* p_endpoint,...);
void svcmgr_unregister(const char* p_endpoint);

#ifdef __cplusplus
}
#endif

#endif /* __SVCMGR_H__ */
